@@ -90,6 +90,8 @@
 #include <asm/sibyte/sb1250_mac.h>
 #include <asm/sibyte/sb1250.h>
 
+#include "sleep.h"
+
 struct cs4297a_state;
 
 static DEFINE_MUTEX(swarm_cs4297a_mutex);
@@ -334,6 +336,7 @@ static int prog_dmabuf_adc(struct cs4297a_state *s)
 	return 0;
 }
 
+
 static int prog_dmabuf_dac(struct cs4297a_state *s)
 {
 	s->dma_dac.ready = 1;
@@ -568,6 +571,7 @@ static void cs_printioctl(unsigned int x)
 }
 #endif
 
+
 static int ser_init(struct cs4297a_state *s)
 {
         int i;
@@ -746,7 +750,7 @@ static int serdma_reg_access(struct cs4297a_state *s, u64 data)
                 /* Since a writer has the DSP open, we have to mux the
                    request in */
                 s->reg_request = data;
-                interruptible_sleep_on(&s->dma_dac.reg_wait);
+		oss_broken_sleep_on(&s->dma_dac.reg_wait, MAX_SCHEDULE_TIMEOUT);
                 /* XXXKW how can I deal with the starvation case where
                    the opener isn't writing? */
         } else {
@@ -788,7 +792,7 @@ static int cs4297a_read_ac97(struct cs4297a_state *s, u32 offset,
         if (serdma_reg_access(s, (0xCLL << 60) | (1LL << 47) | ((u64)(offset & 0x7F) << 40)))
                 return -1;
 
-        interruptible_sleep_on(&s->dma_adc.reg_wait);
+	oss_broken_sleep_on(&s->dma_adc.reg_wait, MAX_SCHEDULE_TIMEOUT);
         *value = s->read_value;
         CS_DBGOUT(CS_AC97, 2,
                   printk(KERN_INFO "cs4297a: rdr reg %x -> %x\n", s->read_reg, s->read_value));
@@ -796,6 +800,7 @@ static int cs4297a_read_ac97(struct cs4297a_state *s, u32 offset,
         return 0;
 }
 
+
 //****************************************************************************
 // "cs4297a_write_ac97()"-- writes an AC97 register
 //****************************************************************************
@@ -825,6 +830,7 @@ static void stop_dac(struct cs4297a_state *s)
 	spin_unlock_irqrestore(&s->lock, flags);
 }
 
+
 static void start_dac(struct cs4297a_state *s)
 {
 	unsigned long flags;
@@ -849,6 +855,7 @@ static void start_dac(struct cs4297a_state *s)
 		  printk(KERN_INFO "cs4297a: start_dac()-\n"));
 }
 
+
 static void stop_adc(struct cs4297a_state *s)
 {
 	unsigned long flags;
@@ -870,6 +877,7 @@ static void stop_adc(struct cs4297a_state *s)
 		  printk(KERN_INFO "cs4297a: stop_adc()-\n"));
 }
 
+
 static void start_adc(struct cs4297a_state *s)
 {
 	unsigned long flags;
@@ -921,6 +929,7 @@ static void start_adc(struct cs4297a_state *s)
 
 }
 
+
 // call with spinlock held! 
 static void cs4297a_update_ptr(struct cs4297a_state *s, int intflag)
 {
@@ -1453,6 +1462,7 @@ static int mixer_ioctl(struct cs4297a_state *s, unsigned int cmd,
 #endif
 		return put_user(s->mix.vol[5], (int *) arg);
 
+
 	case SOUND_MIXER_SYNTH:
 		if (get_user(val, (int *) arg))
 			return -EFAULT;
@@ -1487,6 +1497,7 @@ static int mixer_ioctl(struct cs4297a_state *s, unsigned int cmd,
 #endif
 		return put_user(s->mix.vol[4], (int *) arg);
 
+
 	default:
 		CS_DBGOUT(CS_IOCTL, 4, printk(KERN_INFO
 			"cs4297a: mixer_ioctl(): default\n"));
@@ -1529,6 +1540,7 @@ static int mixer_ioctl(struct cs4297a_state *s, unsigned int cmd,
 	}
 }
 
+
 // --------------------------------------------------------------------- 
 
 static int cs4297a_open_mixdev(struct inode *inode, struct file *file)
@@ -1565,6 +1577,7 @@ static int cs4297a_open_mixdev(struct inode *inode, struct file *file)
 	return nonseekable_open(inode, file);
 }
 
+
 static int cs4297a_release_mixdev(struct inode *inode, struct file *file)
 {
 	struct cs4297a_state *s =
@@ -1574,6 +1587,7 @@ static int cs4297a_release_mixdev(struct inode *inode, struct file *file)
 	return 0;
 }
 
+
 static int cs4297a_ioctl_mixdev(struct file *file,
 			       unsigned int cmd, unsigned long arg)
 {
@@ -1585,6 +1599,7 @@ static int cs4297a_ioctl_mixdev(struct file *file,
 	return ret;
 }
 
+
 // ******************************************************************************************
 //   Mixer file operations struct.
 // ******************************************************************************************
@@ -1598,6 +1613,7 @@ static const struct file_operations cs4297a_mixer_fops = {
 
 // --------------------------------------------------------------------- 
 
+
 static int drain_adc(struct cs4297a_state *s, int nonblock)
 {
         /* This routine serves no purpose currently - any samples
@@ -1642,6 +1658,7 @@ static int drain_dac(struct cs4297a_state *s, int nonblock)
 	return 0;
 }
 
+
 // --------------------------------------------------------------------- 
 
 static ssize_t cs4297a_read(struct file *file, char *buffer, size_t count,
@@ -1725,7 +1742,7 @@ static ssize_t cs4297a_read(struct file *file, char *buffer, size_t count,
 			start_adc(s);
 			if (file->f_flags & O_NONBLOCK)
 				return ret ? ret : -EAGAIN;
-			interruptible_sleep_on(&s->dma_adc.wait);
+			oss_broken_sleep_on(&s->dma_adc.wait, MAX_SCHEDULE_TIMEOUT);
 			if (signal_pending(current))
 				return ret ? ret : -ERESTARTSYS;
 			continue;
@@ -1766,6 +1783,7 @@ static ssize_t cs4297a_read(struct file *file, char *buffer, size_t count,
 	return ret;
 }
 
+
 static ssize_t cs4297a_write(struct file *file, const char *buffer,
 			    size_t count, loff_t * ppos)
 {
@@ -1820,7 +1838,7 @@ static ssize_t cs4297a_write(struct file *file, const char *buffer,
 			start_dac(s);
 			if (file->f_flags & O_NONBLOCK)
 				return ret ? ret : -EAGAIN;
-			interruptible_sleep_on(&d->wait);
+			oss_broken_sleep_on(&d->wait, MAX_SCHEDULE_TIMEOUT);
 			if (signal_pending(current))
 				return ret ? ret : -ERESTARTSYS;
 			continue;
@@ -1883,6 +1901,7 @@ static ssize_t cs4297a_write(struct file *file, const char *buffer,
 	return ret;
 }
 
+
 static unsigned int cs4297a_poll(struct file *file,
 				struct poll_table_struct *wait)
 {
@@ -1942,6 +1961,7 @@ static unsigned int cs4297a_poll(struct file *file,
 	return mask;
 }
 
+
 static int cs4297a_mmap(struct file *file, struct vm_area_struct *vma)
 {
         /* XXXKW currently no mmap support */
@@ -1949,6 +1969,7 @@ static int cs4297a_mmap(struct file *file, struct vm_area_struct *vma)
 	return 0;
 }
 
+
 static int cs4297a_ioctl(struct file *file,
 			unsigned int cmd, unsigned long arg)
 {
@@ -2433,7 +2454,7 @@ static int cs4297a_locked_open(struct inode *inode, struct file *file)
 				return -EBUSY;
 			}
 			mutex_unlock(&s->open_sem_dac);
-			interruptible_sleep_on(&s->open_wait_dac);
+			oss_broken_sleep_on(&s->open_wait_dac, MAX_SCHEDULE_TIMEOUT);
 
 			if (signal_pending(current)) {
                                 printk("open - sig pending\n");
@@ -2450,7 +2471,7 @@ static int cs4297a_locked_open(struct inode *inode, struct file *file)
 				return -EBUSY;
 			}
 			mutex_unlock(&s->open_sem_adc);
-			interruptible_sleep_on(&s->open_wait_adc);
+			oss_broken_sleep_on(&s->open_wait_adc, MAX_SCHEDULE_TIMEOUT);
 
 			if (signal_pending(current)) {
                                 printk("open - sig pending\n");
@@ -2604,15 +2625,12 @@ static int __init cs4297a_init(void)
 	u32 pwr, id;
 	mm_segment_t fs;
 	int rval;
-#ifndef CONFIG_BCM_CS4297A_CSWARM
 	u64 cfg;
 	int mdio_val;
-#endif
 
 	CS_DBGOUT(CS_INIT | CS_FUNCTION, 2, printk(KERN_INFO 
 		"cs4297a: cs4297a_init_module()+ \n"));
 
-#ifndef CONFIG_BCM_CS4297A_CSWARM
         mdio_val = __raw_readq(KSEG1 + A_MAC_REGISTER(2, R_MAC_MDIO)) &
                 (M_MAC_MDIO_DIR|M_MAC_MDIO_OUT);
 
@@ -2638,7 +2656,6 @@ static int __init cs4297a_init(void)
         __raw_writeq(mdio_val | M_MAC_GENC, KSEG1+A_MAC_REGISTER(2, R_MAC_MDIO));
         /* Give the codec some time to finish resetting (start the bit clock) */
         udelay(100);
-#endif
 
 	if (!(s = kzalloc(sizeof(struct cs4297a_state), GFP_KERNEL))) {
 		CS_DBGOUT(CS_ERROR, 1, printk(KERN_ERR
